@@ -2,10 +2,11 @@
 #this repository contains the full copyright notices and license terms.
 import datetime
 import warnings
+from ast import literal_eval
 
 from ..model import ModelView, ModelSQL, fields
 from ..cache import Cache
-from ..tools import safe_eval, datetime_strftime
+from ..tools import datetime_strftime
 from ..transaction import Transaction
 from ..pool import Pool
 from .time_locale import TIME_LOCALE
@@ -62,9 +63,9 @@ class Lang(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        langs = cls.search([('code',) + clause[1:]], order=[])
+        langs = cls.search([('code',) + tuple(clause[1:])], order=[])
         if langs:
-            langs += cls.search([('name',) + clause[1:]], order=[])
+            langs += cls.search([('name',) + tuple(clause[1:])], order=[])
             return [('id', 'in', [l.id for l in langs])]
         return [('name',) + tuple(clause[1:])]
 
@@ -132,7 +133,7 @@ class Lang(ModelSQL, ModelView):
         '''
         for lang in langs:
             try:
-                grouping = safe_eval(lang.grouping)
+                grouping = literal_eval(lang.grouping)
                 for i in grouping:
                     if not isinstance(i, int):
                         raise
@@ -204,26 +205,35 @@ class Lang(ModelSQL, ModelView):
 
     @classmethod
     def create(cls, vlist):
+        pool = Pool()
+        Translation = pool.get('ir.translation')
         # Clear cache
         cls._lang_cache.clear()
-        return super(Lang, cls).create(vlist)
+        languages = super(Lang, cls).create(vlist)
+        Translation._get_language_cache.clear()
+        return languages
 
     @classmethod
-    def write(cls, langs, vals):
+    def write(cls, langs, values, *args):
+        pool = Pool()
+        Translation = pool.get('ir.translation')
         # Clear cache
         cls._lang_cache.clear()
-        super(Lang, cls).write(langs, vals)
+        super(Lang, cls).write(langs, values, *args)
+        Translation._get_language_cache.clear()
 
     @classmethod
     def delete(cls, langs):
         pool = Pool()
         Config = pool.get('ir.configuration')
+        Translation = pool.get('ir.translation')
         for lang in langs:
             if lang.code == Config.get_language():
                 cls.raise_user_error('delete_default')
         # Clear cache
         cls._lang_cache.clear()
         super(Lang, cls).delete(langs)
+        Translation._get_language_cache.clear()
 
     @staticmethod
     def _group(lang, s, monetary=None):
@@ -245,10 +255,10 @@ class Lang(ModelSQL, ModelView):
 
         if monetary:
             thousands_sep = monetary.mon_thousands_sep
-            grouping = safe_eval(monetary.mon_grouping)
+            grouping = literal_eval(monetary.mon_grouping)
         else:
             thousands_sep = lang.thousands_sep
-            grouping = safe_eval(lang.grouping)
+            grouping = literal_eval(lang.grouping)
         if not grouping:
             return (s, 0)
         if s[-1] == ' ':
