@@ -7,7 +7,7 @@ import warnings
 
 from trytond.model import fields
 from trytond.error import WarningErrorMixin
-from trytond.pool import Pool, PoolMeta
+from trytond.pool import Pool, PoolBase
 from trytond.pyson import PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.url import URLMixin
@@ -16,17 +16,17 @@ from trytond.rpc import RPC
 __all__ = ['Model']
 
 
-class Model(WarningErrorMixin, URLMixin):
+class Model(WarningErrorMixin, URLMixin, PoolBase):
     """
     Define a model in Tryton.
     """
-    __metaclass__ = PoolMeta
     _rec_name = 'name'
 
     id = fields.Integer('ID', readonly=True)
 
     @classmethod
     def __setup__(cls):
+        super(Model, cls).__setup__()
         cls.__rpc__ = {
             'default_get': RPC(),
             'fields_get': RPC(),
@@ -35,16 +35,43 @@ class Model(WarningErrorMixin, URLMixin):
             }
         cls._error_messages = {}
 
-        # Copy fields
+        if hasattr(cls, '__depend_methods'):
+            cls.__depend_methods = cls.__depend_methods.copy()
+        else:
+            cls.__depend_methods = collections.defaultdict(set)
+
+        # Copy fields and update depends
         for attr in dir(cls):
             if attr.startswith('_'):
                 continue
-            if isinstance(getattr(cls, attr), fields.Field):
-                setattr(cls, attr, copy.deepcopy(getattr(cls, attr)))
+            if not isinstance(getattr(cls, attr), fields.Field):
+                continue
+            field_name = attr
+            field = copy.deepcopy(getattr(cls, field_name))
+            setattr(cls, field_name, field)
+
+            for attribute in ('on_change', 'on_change_with', 'autocomplete',
+                    'selection_change_with'):
+                if attribute == 'selection_change_with':
+                    if isinstance(
+                            getattr(field, 'selection', None), basestring):
+                        function_name = field.selection
+                    else:
+                        continue
+                else:
+                    function_name = '%s_%s' % (attribute, field_name)
+                function = getattr(cls, function_name, None)
+                if function:
+                    if getattr(function, 'depends', None):
+                        setattr(field, attribute,
+                            getattr(field, attribute) | function.depends)
+                    if getattr(function, 'depend_methods', None):
+                        cls.__depend_methods[(field_name, attribute)] |= \
+                            function.depend_methods
 
     @classmethod
     def __post_setup__(cls):
-        pool = Pool()
+        super(Model, cls).__post_setup__()
 
         # Set _fields
         cls._fields = {}
@@ -72,13 +99,25 @@ class Model(WarningErrorMixin, URLMixin):
             if isinstance(field, (fields.Selection, fields.Reference)) \
                     and not isinstance(field.selection, (list, tuple)) \
                     and field.selection not in cls.__rpc__:
-                cls.__rpc__[field.selection] = RPC()
+                instantiate = 0 if field.selection_change_with else None
+                cls.__rpc__.setdefault(field.selection,
+                    RPC(instantiate=instantiate))
 
             for attribute in ('on_change', 'on_change_with', 'autocomplete'):
                 function_name = '%s_%s' % (attribute, field_name)
-                if getattr(field, attribute, False):
+                if getattr(cls, function_name, None):
                     cls.__rpc__.setdefault(function_name, RPC(instantiate=0))
 
+        # Update depend on methods
+        for (field_name, attribute), others in (
+                cls.__depend_methods.iteritems()):
+            field = getattr(cls, field_name)
+            for other in others:
+                other_field = getattr(cls, other)
+                setattr(field, attribute,
+                    getattr(field, attribute)
+                    | getattr(other_field, attribute))
+
         # Set name to fields
         for name, field in cls._fields.iteritems():
             if field.name is None:
@@ -104,208 +143,26 @@ class Model(WarningErrorMixin, URLMixin):
         """
         Add model in ir.model and ir.model.field.
         """
+        super(Model, cls).__register__(module_name)
         pool = Pool()
         Translation = pool.get('ir.translation')
-        Property = pool.get('ir.property')
+        Model_ = pool.get('ir.model')
+        ModelField = pool.get('ir.model.field')
 
-        cursor = Transaction().cursor
-        # Add model in ir_model
-        cursor.execute("SELECT id FROM ir_model WHERE model = %s",
-                (cls.__name__,))
-        model_id = None
-        if cursor.rowcount == -1 or cursor.rowcount is None:
-            data = cursor.fetchone()
-            if data:
-                model_id, = data
-        elif cursor.rowcount != 0:
-            model_id, = cursor.fetchone()
-        if not model_id:
-            cursor.execute("INSERT INTO ir_model "
-                "(model, name, info, module) VALUES (%s, %s, %s, %s)",
-                (cls.__name__, cls._get_name(), cls.__doc__,
-                    module_name))
-            Property._models_get_cache.clear()
-            cursor.execute("SELECT id FROM ir_model WHERE model = %s",
-                    (cls.__name__,))
-            (model_id,) = cursor.fetchone()
-        elif cls.__doc__:
-            cursor.execute('UPDATE ir_model '
-                'SET name = %s, '
-                    'info = %s '
-                'WHERE id = %s',
-                (cls._get_name(), cls.__doc__, model_id))
+        model_id = Model_.register(cls, module_name)
+        ModelField.register(cls, module_name, model_id)
 
-        # Update translation of model
-        if cls.__doc__:
-            name = cls.__name__ + ',name'
-            src = cls._get_name()
-            cursor.execute('SELECT id FROM ir_translation '
-                'WHERE lang = %s '
-                    'AND type = %s '
-                    'AND name = %s '
-                    'AND (res_id IS NULL OR res_id = %s)',
-                ('en_US', 'model', name, 0))
-            trans_id = None
-            if cursor.rowcount == -1 or cursor.rowcount is None:
-                data = cursor.fetchone()
-                if data:
-                    trans_id, = data
-            elif cursor.rowcount != 0:
-                trans_id, = cursor.fetchone()
-            src_md5 = Translation.get_src_md5(src)
-            if trans_id is None:
-                cursor.execute('INSERT INTO ir_translation '
-                    '(name, lang, type, src, src_md5, value, module, fuzzy) '
-                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                    (name, 'en_US', 'model', src, src_md5, '', module_name,
-                        False))
-            else:
-                cursor.execute('UPDATE ir_translation '
-                    'SET src = %s, src_md5 = %s '
-                    'WHERE id = %s',
-                        (src, src_md5, trans_id))
-
-        # Add field in ir_model_field and update translation
-        cursor.execute('SELECT f.id AS id, f.name AS name, '
-                'f.field_description AS field_description, '
-                'f.ttype AS ttype, f.relation AS relation, '
-                'f.module as module, f.help AS help '
-            'FROM ir_model_field AS f, ir_model AS m '
-            'WHERE f.model = m.id '
-                'AND m.model = %s ',
-            (cls.__name__,))
-        model_fields = {}
-        for field in cursor.dictfetchall():
-            model_fields[field['name']] = field
-
-        # Prefetch field translations
-        if cls._fields:
-            cursor.execute('SELECT id, name, src, type FROM ir_translation '
-                'WHERE lang = %s '
-                    'AND type IN (%s, %s, %s) '
-                    'AND name IN '
-                        '(' + ','.join(('%s',) * len(cls._fields)) + ')',
-                ('en_US', 'field', 'help', 'selection')
-                + tuple([cls.__name__ + ',' + x for x in cls._fields]))
-        trans_fields = {}
-        trans_help = {}
-        trans_selection = {}
-        for trans in cursor.dictfetchall():
-            if trans['type'] == 'field':
-                trans_fields[trans['name']] = trans
-            elif trans['type'] == 'help':
-                trans_help[trans['name']] = trans
-            elif trans['type'] == 'selection':
-                trans_selection.setdefault(trans['name'], {})
-                trans_selection[trans['name']][trans['src']] = trans
-
-        for field_name in cls._fields:
-            field = cls._fields[field_name]
-            relation = ''
-            if hasattr(field, 'model_name'):
-                relation = field.model_name
-            elif hasattr(field, 'relation_name'):
-                relation = field.relation_name
-            if field_name not in model_fields:
-                cursor.execute("INSERT INTO ir_model_field "
-                    "(model, name, field_description, ttype, "
-                        "relation, help, module) "
-                    "VALUES (%s, %s, %s, %s, %s, %s, %s)",
-                    (model_id, field_name, field.string, field._type,
-                        relation, field.help, module_name))
-            elif (model_fields[field_name]['field_description'] != field.string
-                    or model_fields[field_name]['ttype'] != field._type
-                    or model_fields[field_name]['relation'] != relation
-                    or model_fields[field_name]['help'] != field.help):
-                cursor.execute('UPDATE ir_model_field '
-                    'SET field_description = %s, '
-                        'ttype = %s, '
-                        'relation = %s, '
-                        'help = %s '
-                    'WHERE id = %s ',
-                    (field.string, field._type, relation,
-                        field.help, model_fields[field_name]['id']))
-            trans_name = cls.__name__ + ',' + field_name
-            string_md5 = Translation.get_src_md5(field.string)
-            if trans_name not in trans_fields:
-                cursor.execute('INSERT INTO ir_translation '
-                    '(name, lang, type, src, src_md5, value, module, fuzzy) '
-                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                    (trans_name, 'en_US', 'field', field.string,
-                        string_md5, '', module_name, False))
-            elif trans_fields[trans_name]['src'] != field.string:
-                cursor.execute('UPDATE ir_translation '
-                    'SET src = %s, src_md5 = %s '
-                    'WHERE id = %s ',
-                    (field.string, string_md5, trans_fields[trans_name]['id']))
-            help_md5 = Translation.get_src_md5(field.help)
-            if trans_name not in trans_help:
-                if field.help:
-                    cursor.execute('INSERT INTO ir_translation '
-                        '(name, lang, type, src, src_md5, value, module, '
-                            'fuzzy) '
-                        'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                        (trans_name, 'en_US', 'help', field.help, help_md5, '',
-                            module_name, False))
-            elif trans_help[trans_name]['src'] != field.help:
-                cursor.execute('UPDATE ir_translation '
-                    'SET src = %s, src_md5 = %s '
-                    'WHERE id = %s ',
-                    (field.help, help_md5, trans_help[trans_name]['id']))
-            if (hasattr(field, 'selection')
-                    and isinstance(field.selection, (tuple, list))
-                    and ((hasattr(field, 'translate_selection')
-                            and field.translate_selection)
-                        or not hasattr(field, 'translate_selection'))):
-                for (_, val) in field.selection:
-                    if (trans_name not in trans_selection
-                            or val not in trans_selection[trans_name]):
-                        val_md5 = Translation.get_src_md5(val)
-                        cursor.execute('INSERT INTO ir_translation '
-                            '(name, lang, type, src, src_md5, value, module, '
-                                'fuzzy) '
-                            'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                            (trans_name, 'en_US', 'selection', val, val_md5,
-                                '', module_name, False))
-        # Clean ir_model_field from field that are no more existing.
-        for field_name in model_fields:
-            if model_fields[field_name]['module'] == module_name \
-                    and field_name not in cls._fields:
-                #XXX This delete field even when it is defined later
-                # in the module
-                cursor.execute('DELETE FROM ir_model_field '
-                    'WHERE id = %s',
-                    (model_fields[field_name]['id'],))
-
-        # Add error messages in ir_translation
-        cursor.execute('SELECT id, src FROM ir_translation '
-            'WHERE lang = %s '
-                'AND type = %s '
-                'AND name = %s',
-            ('en_US', 'error', cls.__name__))
-        trans_error = {}
-        for trans in cursor.dictfetchall():
-            trans_error[trans['src']] = trans
-
-        errors = cls._get_error_messages()
-        for error in set(errors):
-            if error not in trans_error:
-                error_md5 = Translation.get_src_md5(error)
-                cursor.execute('INSERT INTO ir_translation '
-                    '(name, lang, type, src, src_md5, value, module, fuzzy) '
-                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                    (cls.__name__, 'en_US', 'error', error, error_md5, '',
-                        module_name, False))
+        Translation.register_model(cls, module_name)
+        Translation.register_fields(cls, module_name)
+        Translation.register_error_messages(cls, module_name)
 
     @classmethod
-    def _get_error_messages(cls):
-        return cls._error_messages.values()
-
-    @classmethod
-    def default_get(cls, fields_names, with_rec_name=True):
+    def default_get(cls, fields_names, with_rec_name=True,
+            with_on_change=True):
         '''
         Return a dict with the default values for each field in fields_names.
         If with_rec_name is True, rec_name will be added.
+        If with_on_change is True, on_change will be added.
         '''
         pool = Pool()
         Property = pool.get('ir.property')
@@ -329,7 +186,8 @@ class Model(WarningErrorMixin, URLMixin):
                     value[field_name + '.rec_name'] = Target(
                         value[field_name]).rec_name
 
-        value = cls._default_on_change(value)
+        if with_on_change:
+            value = cls._default_on_change(value)
         if not with_rec_name:
             for field in value.keys():
                 if field.endswith('.rec_name'):
@@ -368,6 +226,7 @@ class Model(WarningErrorMixin, URLMixin):
         pool = Pool()
         Translation = pool.get('ir.translation')
         FieldAccess = pool.get('ir.model.field.access')
+        ModelAccess = pool.get('ir.model.access')
 
         #Add translation to cache
         language = Transaction().language
@@ -418,10 +277,15 @@ class Model(WarningErrorMixin, URLMixin):
                     'loading',
                     'filename',
                     'selection_change_with',
+                    'domain',
                     ):
                 if getattr(cls._fields[field], arg, None) is not None:
-                    res[field][arg] = copy.copy(getattr(cls._fields[field],
-                        arg))
+                    value = getattr(cls._fields[field], arg)
+                    if isinstance(value, set):
+                        value = list(value)
+                    else:
+                        value = copy.copy(value)
+                    res[field][arg] = value
             if not accesses.get(field, {}).get('write', True):
                 res[field]['readonly'] = True
                 if res[field].get('states') and \
@@ -432,9 +296,9 @@ class Model(WarningErrorMixin, URLMixin):
                         and getattr(cls._fields[field], arg):
                     res[field][arg] = copy.copy(getattr(cls._fields[field],
                         arg))
-            if isinstance(cls._fields[field],
-                    (fields.Function, fields.One2Many)) \
-                    and not cls._fields[field].order_field:
+            if (isinstance(cls._fields[field],
+                        (fields.Function, fields.One2Many))
+                    and not getattr(cls, 'order_%s' % field, None)):
                 res[field]['sortable'] = False
             if ((isinstance(cls._fields[field], fields.Function)
                     and not cls._fields[field].searcher)
@@ -490,16 +354,23 @@ class Model(WarningErrorMixin, URLMixin):
                     relation = copy.copy(
                         cls._fields[field].get_target().__name__)
                 res[field]['relation'] = relation
-                res[field]['domain'] = copy.copy(cls._fields[field].domain)
                 res[field]['context'] = copy.copy(cls._fields[field].context)
                 res[field]['create'] = accesses.get(field, {}).get('create',
                     True)
                 res[field]['delete'] = accesses.get(field, {}).get('delete',
                     True)
             if res[field]['type'] == 'one2many' \
-                    and hasattr(cls._fields[field], 'field'):
+                    and getattr(cls._fields[field], 'field', None):
                 res[field]['relation_field'] = copy.copy(
                         cls._fields[field].field)
+            if res[field]['type'] == 'many2one':
+                target = cls._fields[field].get_target()
+                for target_name, target_field in target._fields.iteritems():
+                    if (target_field._type == 'one2many'
+                            and target_field.model_name == cls.__name__
+                            and target_field.field == field):
+                        res[field]['relation_field'] = target_name
+                        break
             if res[field]['type'] in ('datetime', 'time'):
                 res[field]['format'] = copy.copy(cls._fields[field].format)
             if res[field]['type'] == 'selection':
@@ -519,11 +390,13 @@ class Model(WarningErrorMixin, URLMixin):
                 if attr in res[field]:
                     res[field][attr] = encoder.encode(res[field][attr])
 
-        if fields_names:
+        for i in res.keys():
             # filter out fields which aren't in the fields_names list
-            for i in res.keys():
+            if fields_names:
                 if i not in fields_names:
                     del res[i]
+            elif not ModelAccess.check_relation(cls.__name__, i, mode='read'):
+                del res[i]
         return res
 
     def on_change_with(self, fieldnames):
@@ -602,6 +475,16 @@ class Model(WarningErrorMixin, URLMixin):
             return False
         return (self.__name__, self.id) == (other.__name__, other.id)
 
+    def __lt__(self, other):
+        if not isinstance(other, Model) or self.__name__ != other.__name__:
+            return NotImplemented
+        return self.id < other.id
+
+    # TODO: replace by total_ordering when 2.6 will be dropped
+    __gt__ = lambda self, other: not (self < other or self == other)
+    __le__ = lambda self, other: self < other or self == other
+    __ge__ = lambda self, other: not self < other
+
     def __ne__(self, other):
         if not isinstance(other, Model):
             return NotImplemented
