@@ -293,10 +293,11 @@ class ModelStorage(Model):
                 or isinstance(f, fields.Property))]
         ids = map(int, records)
         datas = cls.read(ids, fields_names=fields_names)
+        datas = dict((d['id'], d) for d in datas)
         field_defs = cls.fields_get(fields_names=fields_names)
         to_create = []
-        for data in datas:
-            data = convert_data(field_defs, data)
+        for id in ids:
+            data = convert_data(field_defs, datas[id])
             to_create.append(data)
         new_records = cls.create(to_create)
         new_ids = dict(izip(ids, map(int, new_records)))
@@ -885,7 +886,7 @@ class ModelStorage(Model):
         def validate_domain(field):
             if not field.domain:
                 return
-            if field._type == 'dict':
+            if field._type in ['dict', 'reference']:
                 return
             if field._type in ('many2one', 'one2many'):
                 Relation = pool.get(field.model_name)
