@@ -3,8 +3,7 @@
 from trytond.protocols.sslsocket import SSLSocket
 from trytond.protocols.dispatcher import dispatch
 from trytond.config import CONFIG
-from trytond.protocols.common import daemon, GZipRequestHandlerMixin, \
-    RegisterHandlerMixin
+from trytond.protocols.common import daemon, RegisterHandlerMixin
 from trytond.exceptions import UserError, UserWarning, NotLogged, \
     ConcurrencyException
 from trytond import security
@@ -50,12 +49,13 @@ def dump_time(self, value, write):
         'hour': value.hour,
         'minute': value.minute,
         'second': value.second,
+        'microsecond': value.microsecond,
         }
     self.dump_struct(value, write)
 
 xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
 xmlrpclib.Marshaller.dispatch[type(None)] = \
-        lambda self, value, write: self.dump_bool(bool(value), write)
+        lambda self, value, write: write("<value><nil/></value>")
 xmlrpclib.Marshaller.dispatch[datetime.date] = dump_date
 xmlrpclib.Marshaller.dispatch[datetime.time] = dump_time
 xmlrpclib.Marshaller.dispatch[buffer] = dump_buffer
@@ -85,7 +85,8 @@ def end_struct(self, data):
         if dct['__class__'] == 'date':
             dct = datetime.date(dct['year'], dct['month'], dct['day'])
         elif dct['__class__'] == 'time':
-            dct = datetime.time(dct['hour'], dct['minute'], dct['second'])
+            dct = datetime.time(dct['hour'], dct['minute'], dct['second'],
+                dct['microsecond'])
         elif dct['__class__'] == 'Decimal':
             dct = Decimal(dct['decimal'])
     self._stack[mark:] = [dct]
@@ -118,8 +119,8 @@ class GenericXMLRPCRequestHandler:
                 if object_type == 'system' and method == 'getCapabilities':
                     return {
                         'introspect': {
-                            'specUrl': 'http://xmlrpc-c.sourceforge.net/' \
-                                    'xmlrpc-c/introspection.html',
+                            'specUrl': ('http://xmlrpc-c.sourceforge.net/'
+                                'xmlrpc-c/introspection.html'),
                             'specVersion': 1,
                         },
                     }
@@ -145,8 +146,7 @@ class GenericXMLRPCRequestHandler:
             security.logout(database_name, user, session)
 
 
-class SimpleXMLRPCRequestHandler(GZipRequestHandlerMixin,
-        RegisterHandlerMixin,
+class SimpleXMLRPCRequestHandler(RegisterHandlerMixin,
         GenericXMLRPCRequestHandler,
         SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
     protocol_version = "HTTP/1.1"
@@ -179,9 +179,8 @@ class SimpleXMLRPCRequestHandler(GZipRequestHandlerMixin,
 class SecureXMLRPCRequestHandler(SimpleXMLRPCRequestHandler):
 
     def setup(self):
-        self.connection = SSLSocket(self.request)
-        self.rfile = socket._fileobject(self.request, "rb", self.rbufsize)
-        self.wfile = socket._fileobject(self.request, "wb", self.wbufsize)
+        self.request = SSLSocket(self.request)
+        SimpleXMLRPCRequestHandler.setup(self)
 
 
 class SimpleThreadedXMLRPCServer(SocketServer.ThreadingMixIn,
@@ -203,21 +202,9 @@ class SimpleThreadedXMLRPCServer(SocketServer.ThreadingMixIn,
 
     def server_close(self):
         SimpleXMLRPCServer.SimpleXMLRPCServer.server_close(self)
-        for handler in self.handlers:
+        for handler in self.handlers.copy():
             self.shutdown_request(handler.request)
 
-    if sys.version_info[:2] <= (2, 6):
-
-        def shutdown_request(self, request):
-            """Called to shutdown and close an individual request."""
-            try:
-                #explicitly shutdown.  socket.close() merely releases
-                #the socket and waits for GC to perform the actual close.
-                request.shutdown(socket.SHUT_WR)
-            except socket.error:
-                pass  # some platforms may raise ENOTCONN here
-            self.close_request(request)
-
 
 class SimpleThreadedXMLRPCServer6(SimpleThreadedXMLRPCServer):
     address_family = socket.AF_INET6
@@ -228,8 +215,7 @@ class SecureThreadedXMLRPCServer(SimpleThreadedXMLRPCServer):
     def __init__(self, server_address, HandlerClass, logRequests=1):
         SimpleThreadedXMLRPCServer.__init__(self, server_address, HandlerClass,
                 logRequests)
-        self.socket = SSLSocket(socket.socket(self.address_family,
-            self.socket_type))
+        self.socket = socket.socket(self.address_family, self.socket_type)
         self.server_bind()
         self.server_activate()
 
